#include "main.h"

void KeyDown(SDL_KeyboardEvent *key)
{
	switch(key->keysym.sym){
		case SDLK_LEFT:
			left_pressed = true;
			break;
		case SDLK_RIGHT:
			right_pressed = true;
			break;
		case SDLK_UP:
			up_pressed = true;
			break;
		case SDLK_DOWN:
			down_pressed = true;
			break;
	}
}

void KeyUp(SDL_KeyboardEvent *key)
{
	switch(key->keysym.sym){
		case SDLK_LEFT:
			left_pressed = false;
			xvel = 0;
			break;
		case SDLK_RIGHT:
			right_pressed = false;
			xvel = 0;
			break;
		case SDLK_UP:
			up_pressed = false;
			break;
		case SDLK_DOWN:
			down_pressed = false;
			break;
		case SDLK_SPACE:
			if(activeObject != NULL) {
				activeObject->action();
			}
			break;
		case SDLK_ESCAPE:
			running = false;
			break;
	}
}

void Move()
{
	moved = false;
	bool onthefloor = ypos + plimage->h >= HEIGHT;
	// Gravity
	if (!onthefloor)
	{
		yvel += 2;
		moved = true;
	}
	else yvel = 0;
	
	// Jump
	if (up_pressed) 
	{
		if(onthefloor) yvel -= 25;
		else yvel -= 1;
		moved = true;
	}
	

	if (down_pressed) yvel += 1;
	if (right_pressed)
	{
		xvel += 1;
		moved = true;
	}
	if (left_pressed)
	{
		xvel -= 1;
		moved = true;
	}
	
	// X-axis: checking for scrolling
	// Right border
	int rboundary = (int) 7 * WIDTH / 8 - plimage->w;
	int lboundary = (int) WIDTH / 8;
	int maxbgpos = bgimage->w - WIDTH;
	if (xpos >= rboundary && xvel > 0 && bgpos < maxbgpos)
	{
		xpos = rboundary;
		bgpos += xvel;
		if (bgpos > maxbgpos) {
			bgpos = maxbgpos;
		}
	}
	// Left border
	else if (xpos <= lboundary && xvel < 0 && bgpos > 0)
	{
		xpos = lboundary;
		bgpos += xvel;
		if (bgpos < 0) {
			bgpos = 0;
		}
	}
	// No border touched
	else
	{
		int maxxpos = WIDTH - plimage->w;
		xpos += xvel;
		if (xpos > maxxpos) xpos = maxxpos;
		if (xpos < 0) xpos = 0;
	}
		
	ypos += yvel;
	
	// Y-axis: collision checking
	if (ypos < INFOBARHEIGHT) ypos = INFOBARHEIGHT;
	if (ypos > (HEIGHT - plimage->h)) ypos = HEIGHT - plimage->h;
}

void RedrawPanel()
{
    SDL_Surface *resulting_text;
	char infostring[1024];
	if (activeObject == NULL) {
		sprintf(infostring, "Level %u, Life: %u/%u", player.level, player.life, player.maxlife);
	} else {
		char *ptr;
		ptr = activeObject->showInfo();
		strcpy(infostring, ptr);
	}
		
	
	SDL_Color fontcolor = {0,0,0,255};
	
	resulting_text = TTF_RenderText_Blended(font, infostring, fontcolor);
	
	dest.x = WIDTH - resulting_text->w - 10;
	dest.y = 10;
	dest.w = 400;
	dest.h = 50;
	
	SDL_BlitSurface(resulting_text, NULL, screen, &dest);
	SDL_FreeSurface(resulting_text);
}

void RedrawBackground()
{
	dest.x = bgpos;
	dest.y = 0;
	dest.w = WIDTH;
	dest.h = HEIGHT;
	
	SDL_BlitSurface(bgimage, &dest, screen, NULL);
}
void RedrawPlayer()
{
	dest.x = xpos;
	dest.y = ypos;
	dest.w = plimage->w;
	dest.h = plimage->h;
	
	SDL_BlitSurface(plimage, NULL, screen, &dest);
}

void RedrawEntrances()
{
	int i;
	int bgstop = bgpos + WIDTH;
	for (i = 0;i < MAXOBJECTS; i++) {
		MapObject* currentObject = LevelContainer::getInstance()->currentLevel->mapobjects[i];
		if (currentObject != NULL) {
			int start = currentObject->start;
			int stop = currentObject->stop;
			if ((bgpos <= start) && (start <= bgstop) || (bgpos <= stop) && (stop <= bgstop)) {
				dest.x = currentObject->start - bgpos;
				dest.y = HEIGHT - currentObject->image->h;
				dest.w = plimage->w;
				dest.h = plimage->h;
				SDL_BlitSurface(currentObject->image, NULL, screen, &dest);
			}
		}
	}
}

void Redraw()
{
	RedrawBackground();
	RedrawEntrances();
	RedrawPlayer();
	RedrawPanel();
	SDL_Flip(screen);
}

void CheckActiveObject()
{
	int i;
	int playerpos = (int) bgpos + xpos + plimage->w / 2;
	activeObject = NULL;
	
	for (i = 0;i < MAXOBJECTS; i++) {
		MapObject* currentObject = LevelContainer::getInstance()->currentLevel->mapobjects[i];
		if (currentObject != NULL) {
			if ((currentObject->start <= playerpos) && (playerpos <= currentObject->stop)) {
				activeObject = currentObject;
			}
		}
	}
}


bool Initialize()
{
	printf("Initializing SDL.\n");
	if (SDL_Init(SDL_INIT_VIDEO) != 0) {
		fprintf(stderr, "Unable to initialize SDL: %s\n", SDL_GetError());
		return false;
	}
	
	printf("Initializing window.\n");
	screen = SDL_SetVideoMode(WIDTH, HEIGHT, 0, SDL_ANYFORMAT);
	if (screen == NULL) {
		fprintf(stderr, "Unable to set video mode: %s\n", SDL_GetError());
		return false;
	}
	SDL_WM_SetCaption(TITLE, TITLE);
	
	printf("Loading levels.\n");
	LevelContainer::getInstance()->Initialize();
	
	printf("Loading images.\n");	
	plimage = LoadImage(PLAYER);
	bgimage = LoadImage(LevelContainer::getInstance()->currentLevel->background);	
	
	printf("Initializing fonts.\n");
	if (TTF_Init() != 0) 
	{
		printf("Unable to initialize SDL_ttf: %s \n", TTF_GetError());
		return false;
	}

	font = TTF_OpenFont(FONT, FONTSIZE);
	if (font == NULL){
		printf("Unable to load font: %s %s \n", FONT, TTF_GetError());
		return false;
	}
	
	xpos = 0;
	ypos = HEIGHT - plimage->h;
	
	Redraw();
	movingEnabled = true;
	return true;
}

int main()
{
	running = true;
	if(Initialize()) {
		while(running) {
			while(SDL_PollEvent(&event)) {
				switch(event.type) {
					case SDL_KEYDOWN:
						KeyDown(&event.key);
						break;
					case SDL_KEYUP:
						KeyUp(&event.key);
						break;
					case SDL_QUIT:
						running = false;
						break;
				}
			}
			if (movingEnabled) {
				Move();
			}
			if (moved) {
				Redraw();
				CheckActiveObject();
				if(activeObject != NULL) {
					activeObject->showInfo();
				}
			}
			SDL_Delay(30);
		}
	}
}